import time

from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from webdriver_manager.chrome import ChromeDriverManager
from pathlib import Path

# CLICK_TYPE configures which key is used to open the post in a new tab
CLICK_TYPE = Keys.COMMAND # Use this if you are on a Mac
# CLICK_TYPE = Keys.CONTROL # Use this if you are on a PC 

class Craigslist:
    def __init__(self, username):
        self.base_url = 'https://accounts.craigslist.org'
        options = Options()
        current_directory = Path().absolute()
        options.add_argument(f"user-data-dir={current_directory}/sessions/{username}")
        options.add_argument('--headless')
        options.add_argument("--window-size=1920,1080")
        options.add_argument("--nogpu")
        options.add_argument("--disable-gpu")
        options.add_argument("user-agent=Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36")
        options.add_argument("--no-sandbox")
        options.add_argument("--enable-javascript")
        options.add_experimental_option("excludeSwitches", ["enable-automation"])
        options.add_experimental_option('useAutomationExtension', False)
        options.add_argument('--disable-blink-features=AutomationControlled')
        self.driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), chrome_options=options)

    def get_login_page(self):
        self.driver.get(f'{self.base_url}/login?rp=%2Flogin%2Fhome&rt=L')
    
    def login(self, username, password):
        email_box = self.driver.find_element(By.ID, 'inputEmailHandle')
        password_box = self.driver.find_element(By.ID, 'inputPassword')
        email_box.send_keys(username)
        password_box.send_keys(password)
        login_button = self.driver.find_element(By.CLASS_NAME, 'accountform-btn')
        login_button.click()
    
    def get_email_verify(self):
        self.driver.find_element(By.CLASS_NAME, 'submit-onetime-link-button')
        
    def check_logged_in(self):
        self.get_login_page()
        return self.driver.current_url == f"{self.base_url}/login/home"
            
    def filter_active_posts(self):
        active_button = self.driver.find_element(By.NAME, "filter_active")
        active_button.click()

    def set_home_window(self):
        self.home_window = self.driver.current_window_handle

    def get_posts(self):
        return self.driver.find_elements(By.CSS_SELECTOR, "tr[class^='posting-row']")
    
    def get_post_date(self, post):
        return post.find_element(By.TAG_NAME, "time").text

    def get_post_title(self, post):
        return post.find_element(By.CLASS_NAME, "title").text

    def command_click(self, element):
        ActionChains(self.driver) \
            .key_down(CLICK_TYPE) \
            .click(element) \
            .key_up(CLICK_TYPE) \
            .perform()
    
    def delete_post(self, post):
        delete_button = post.find_element(
            By.CSS_SELECTOR, "form[class^='manage delete']")
        self.command_click(delete_button)

    def contains_button_click(self, type, class_contains, value_contains):
        time.sleep(2)
        self.driver.find_element(
            By.XPATH, f"//{type}[contains(@class, '{class_contains}') and contains(@value, '{value_contains}')]").click()
        
    def repost(self, post):
        self.delete_post(post)
        self.driver.switch_to.window(self.driver.window_handles[1])
        self.contains_button_click('input', 'managebtn', 'Repost this Posting')
        self.contains_button_click('button', 'submit-button', 'continue')
        try:
            self.contains_button_click('button', 'button', 'Continue')
        except:
            self.contains_button_click('button', 'continue', 'continue')
            self.driver.find_element(By.XPATH, "//*[@id='leafletForm']/button[1]").click()
            time.sleep(2)
            self.contains_button_click('button', 'button', 'Continue')

    def get_renewal_buttons(self):
        return self.driver.find_elements(By.XPATH, "//input[contains(@class, 'managebtn') and contains(@value, 'renew')]")



import os
import random
import sys
import time
from datetime import datetime
from email import message_from_bytes

import requests
from bs4 import BeautifulSoup
from craigslist import Craigslist
from craigslist_email import CraigslistEmail
from selenium.webdriver.common.by import By

# SET UP ENVIRONMENT VARIABLES
CL_USERNAME = os.environ.get('CL_USER')
CL_PASSWORD = os.environ.get('CL_PASS')
EMAIL_ADDRESS = CL_USERNAME
EMAIL_PASSWORD = os.environ.get('CL_EMAIL_PASS')
# WEBHOOK (optional) configures the discord webhook url to send summary to
WEBHOOK = os.environ.get('DISCORD_WEBHOOK')

# MAX_POST_DAYS configures the maximum number of days after posting before reposting 
MAX_POST_DAYS = random.randint(8,14)
# MAX_REPOSTS configures the maximum number of posts to repost per run
MAX_REPOSTS = int(sys.argv[1])
# MAX_RENEWALS configures the manimum number of posts to renew per run
MAX_RENEWALS = int(sys.argv[2])

# IMAP_SERVER for email provider, default for gmail
IMAP_SERVER = 'imap.gmail.com'

repost_count = 0
renew_count = 0

current_time = datetime.now()

# Initialize connection to email inbox
inbox = CraigslistEmail(IMAP_SERVER, EMAIL_ADDRESS, EMAIL_PASSWORD)
# Delete unread emails from robot@craigslist.com prior to beginning craigslist portion of script
inbox.delete_craigslist_emails()

# Initialize Craigslist object and login to craigslist account
cl = Craigslist(CL_USERNAME)

# Check if logged in, if not, wait for email confirmation and verify login
if not cl.check_logged_in():
    cl.login(CL_USERNAME, CL_PASSWORD)
    time.sleep(1)
    if not cl.check_logged_in():
        print('Not logged in, verifying login')
        cl.driver.find_element(By.CLASS_NAME, 'submit-onetime-link-button').click()
        time.sleep(5)
        # Wait for emails from craigslist to be received
        for msgid, data in inbox.wait_unread_emails():
            print('Found email from craigslist')
            contents = message_from_bytes(data[b'RFC822'])
            content = contents.get_payload()[1].get_payload(decode=True)
            soup = BeautifulSoup(content, 'html.parser')
            links = soup.find_all('a', href=True)
            # When email is received, get the link to verify login
            cl.driver.get(links[0]['href'])
            inbox.email.delete_messages(msgid)
            time.sleep(1)
cl.filter_active_posts()
time.sleep(2)

cl.set_home_window()
active_posts = cl.get_posts()

# Loop through active posts and repost if older than MAX_POST_DAYS
for post in active_posts:
    if repost_count == MAX_REPOSTS:
        print(f"Maximum reposts of {MAX_REPOSTS} reached")
        break
    # parse_date converts string from dd MTH yyyy HH:MM format to datetime object
    def parse_date(string):    
        date = string.split(' ')
        time = date[3].split(':')
        return datetime(int(date[2]), datetime.strptime(date[1], "%b").month, int(date[0]), int(time[0]), int(time[1]))
    post_date = parse_date(cl.get_post_date(post))
    posted_days = (current_time - post_date).days
    if posted_days > MAX_POST_DAYS:
        repost_count += 1
        print(f'Reposting, post active for {posted_days} days: {cl.get_post_title(post)}')
        cl.repost(post)
        cl.driver.close()
        cl.driver.switch_to.window(cl.home_window)
        sleep_time = random.randint(10,230)
        print(f'Reposted, sleeping {sleep_time} seconds...')
        time.sleep(sleep_time)

# Delete unread emails which are verifying reposts
inbox.delete_craigslist_emails()
print('Unread emails from craigslist deleted')

# Loop through present renew buttons, clicking to renew post
cl.driver.refresh()
for button in cl.get_renewal_buttons():
    if renew_count == MAX_RENEWALS:
        print(f"Maximum renewals of {MAX_RENEWALS} reached")
        break
    renew_count += 1
    sleep_time = random.uniform(0.5,3.5)
    print(f'Renewing post and sleeping for {sleep_time} seconds...')
    cl.command_click(button)
    time.sleep(sleep_time)

cl.driver.quit()

# Print and send count of posts resposted and renewed to discord webhook
summary = f'{CL_USERNAME}\nCraigslist.com listings renewed: {renew_count}\nCraigslist.com listings reposted: {repost_count}'
print(summary)
if WEBHOOK:
    data = {'content':summary}
    result = requests.post(WEBHOOK, json = data)


import time

from imapclient import IMAPClient


class CraigslistEmail:
    def __init__(self, host, email, password):
        self.email = IMAPClient(host, use_uid=True, ssl=True)
        self.email.login(email, password)
        print('Connected to email inbox via IMAP')
        
    def get_unread_emails(self):
        self.email.select_folder('INBOX')
        return self.email.search(['UNSEEN', 'FROM', 'robot@craigslist.org'])

    def wait_unread_emails(self):
        print('Fetching unread emails from robot@craigslist.org')
        emails = self.get_unread_emails()
        while len(emails) == 0:
            print('No emails found, waiting 15 seconds')
            time.sleep(15)
            emails = self.get_unread_emails()
        return self.email.fetch(emails, 'RFC822').items()
    
    def delete_craigslist_emails(self):
        self.email.delete_messages(self.get_unread_emails())